# -*- coding: utf-8 -*- 
# @Time    : 03/22/2024 
# @Author  : nzooherd 
# @File    : payment_service.py 
# @Software: Visual Studio Code


from logging import getLogger
from sqlalchemy.exc import IntegrityError
from component.handle_context import HandleContext
from dao import transaction
from dao.model import Purchase, Redeem
from dao.payment_dao import PaymentDao
from dao.purchase import PurchaseDao
from dao.redeem_dao import RedeemDao
from dao.video_dao import VideoDao
from exception import InvalidRedeemException
from template.account import AccountTemplate
from template.redeem import RedeemInvalidTemplate, RedeemSuccessTemplate, RedeemUsedTemplate
from template.topup import TopupTemplate
from utills.redeem_code import RedeemCode


logger = getLogger()

class PaymentService:

    def __init__(self) -> None:
        self.payment_dao = PaymentDao()
        self.video_dao = VideoDao()
        self.redeem_dao = RedeemDao()
        self.purchase_dao = PurchaseDao()
        
        self.handle_context = HandleContext()


    async def account_info(self) -> AccountTemplate:
        user_id = self.handle_context.get_user_id()
        user_payment = await self.payment_dao.query_by_user_id(user_id)
        return AccountTemplate(user_payment.amount)

    @transaction
    async def active_gift_card(self, gift_card: str) -> RedeemSuccessTemplate:
        try:
            user_id = self.handle_context.get_user_id()
            value =  RedeemCode.decode(gift_card)
            if (value <= 0):
                raise InvalidRedeemException("Invalid gift card code")

            payment = await self.payment_dao.modify_amount(user_id, value)
            redeem = Redeem(user_id=user_id, gift_code=gift_card)
            await self.redeem_dao.insert(redeem)
            logger.info(f"User: {user_id} active gift card: {gift_card}")
            return RedeemSuccessTemplate(value, payment.amount)
        except IntegrityError:
            logger.warn(f"Active gift card has beed used: {gift_card}")
            return RedeemUsedTemplate(gift_card)
        except InvalidRedeemException:
            logger.warn(f"Invalid gift card code: {gift_card}")
            return RedeemInvalidTemplate(gift_card)


    @transaction
    async def purchase_video(self, video_md5: str):
        user_id = self.handle_context.get_user_id()
        video_id = (await self.video_dao.get_video_by_md5(video_md5)).id
        logger.info(f"User: {user_id} purchase video: {video_id}")
        try:
            await self.purchase_dao.insert(Purchase(user_id=user_id, video_id=video_id))
            await self.payment_dao.modify_amount(user_id, -1)
        except IntegrityError:
            logger.warn(f"User: {user_id} already purchase video: {video_id}")
    
    async def purchase_giftcard_prompt(self, video_md5: str):
        return TopupTemplate()
    